/**
 * @file          modules/sfdx-falcon-notifications/index.ts
 * @copyright     Vivek M. Chawla - 2018
 * @author        Vivek M. Chawla <@VivekMChawla>
 * @version       1.0.0
 * @license       MIT
 * @requires      module:debug
 * @requires      module:???
 * @summary       SFDX-Falcon helper module for providing notifications
 * @description   ???
 */
/// <reference types="node" />
import { SfdxFalconResult } from '../sfdx-falcon-result';
import { Subscriber } from '../sfdx-falcon-types';
/**
 * @class       FalconProgressNotifications
 * @description Manages progress notifications inside Falcon
 * @version     1.0.0
 * @public
 */
export declare class FalconProgressNotifications {
    /**
     * @method      start
     * @param       {string}  message Required. The baseline message that will
     *              be used for each interval-based push (ie. Observer.next()).
     * @param       {number}  interval  Requiredl The interval in milliseconds.
     *              Use 1000 if you want a per-second progress count.
     * @param       {SfdxFalconResult}  result  Required. The SfdxFalconResult
     *              that will be used to pull in the elapsed time used by the
     *              Progress Notification function.
     * @param       {Subscriber}  observer  Required. Subscriber to an Observable
     *              object.
     * @returns     {NodeJS.Timeout}  Result of a call to setTimeout().
     * @description Starts a progress notification interval timeout that is able
     *              to provide regular updates to an Observable object.
     * @public @static
     */
    static start(message: string, interval: number, result: SfdxFalconResult, observer: Subscriber): NodeJS.Timeout;
    /**
     * @method      finish
     * @param       {NodeJS.Timeout}  timeoutObj  Required. The timeout that will
     *              be cleared.
     * @returns     {void}
     * @description Given a Timeout object (ie. the thing that's returned from a
     *              call to setInterval() or setTimeout()), clears that timeout
     *              so that it doesn't execute (or execute again).
     * @public @static
     */
    static finish(timeoutObj: NodeJS.Timeout): void;
    /**
     * @method      killAll
     * @returns     {void}
     * @description Kills (calls clearInterval()) on ALL of the Timeout Refs that
     *              have been created as part of the SFDX-Falcon notification
     *              system.
     * @public @static
     */
    static killAll(): void;
    /** Holds a reference to every timeout object created as part of the SFDX-Falcon notification process. */
    private static timeoutRefs;
}
/**
 * @function    updateObserver
 * @param       {Subscriber}  observer  Required. Subscriber to an Observable object. Does nothing
 *              if typeof observer.next is undefined.
 * @param       {string}  message Required. The message to be passed to observer.next().
 * @returns     {void}
 * @description Posts the provided message to observer.next() ONLY if an Observer was provided.
 */
export declare function updateObserver(observer: Subscriber, message: string): void;
