import { SfdxFalconResult } from '../../../../sfdx-falcon-result';
import { AppxEngineActionContext } from '../../appx';
import { AppxEngineAction } from '../../appx/actions';
import { ActionOptions } from '../../../types/';
/**
 * @interface   ActionResultDetail
 * @extends     SfdxCliActionResultDetail
 * @description Represents the structure of the "Result Detail" object used by this ACTION.
 */
/**
 * @class       DeployMetadataAction
 * @extends     AppxEngineAction
 * @description Implements the action "deploy-metadata".
 * @public
 */
export declare class DeployMetadataAction extends AppxEngineAction {
    /**
     * @method      initializeAction
     * @returns     {void}
     * @description Sets member variables based on the specifics of this action.
     * @protected
     */
    protected initializeAction(): void;
    /**
     * @method      validateActionOptions
     * @param       {ActionOptions} actionOptions Required. The options that
     *              should be validated because they are required by this specific
     *              action.
     * @returns     {void}
     * @description Given an object containing Action Options, make sure that
     *              everything expected by this Action in order to properly
     *              execute has been provided.
     * @protected
     */
    protected validateActionOptions(actionOptions: ActionOptions): void;
    /**
     * @method      executeAction
     * @param       {ActionOptions} [actionOptions] Optional. Any options that the
     *              command execution logic will require in order to properly do
     *              its job.
     * @returns     {Promise<SfdxFalconResult>} Resolves with an SfdxFalconResult
     *              of type ACTION that has one or more EXECUTOR Results as
     *              children.
     * @description Performs the custom logic that's wrapped by the execute method
     *              of the base class.
     * @protected @async
     */
    protected executeAction(actionContext: AppxEngineActionContext, actionOptions?: ActionOptions): Promise<SfdxFalconResult>;
}
