import { SfdxFalconRecipe } from '../../../sfdx-falcon-recipe';
import { SfdxFalconRecipeJson } from '../../../sfdx-falcon-recipe';
import { AppxRecipeEngine } from '../../../sfdx-falcon-recipe/engines/appx';
import { ListrContext } from '../../../sfdx-falcon-types';
import { CompileOptions } from '../../types';
import { ExecutionOptions } from '../../types';
export interface AppxDemoConfigCompileOptions {
    targetOrgAlias: string;
    haltOnError: boolean;
    skipOrgRefresh: boolean;
    skipGroups: string[];
    skipActions: string[];
}
/**
 * @class       AppxDemoConfigEngine
 * @extends     AppxRecipeEngine
 * @summary     Implementation for an ADK specific AppX Recipe Engine.
 * @description Implementation for an ADK specific AppX Recipe Engine.
 * @public
 */
export declare class AppxDemoConfigEngine extends AppxRecipeEngine {
    /**
     * @method      compileRecipe
     * @param       {SfdxFalconRecipe}  recipe  Required.
     * @param       {CompileOptions}  compileOptions  Required.
     * @returns     {Promise<AppxDemoConfigEngine>} Resolves with a fully
     *              constructed AppxDemoConfigEngine. Implemented this way so
     *              we can use asynchronous code while constructing the object.
     * @description ???
     * @public @static @async
     */
    static compileRecipe(recipe: SfdxFalconRecipe, compileOptions: CompileOptions): Promise<AppxDemoConfigEngine>;
    /**
     * @method      validateRecipe
     * @param       {SfdxFalconRecipeJson}  sfdxFalconRecipeJson  Required.
     * @returns     {void}  ???
     * @description ???
     * @version     1.0.0
     * @public @static
     */
    static validateRecipe(sfdxFalconRecipeJson: SfdxFalconRecipeJson): void;
    /**
     * @method      validateInnerRecipe
     * @param       {SfdxFalconRecipeJson}  sfdxFalconRecipeJson  Required.
     * @returns     {Promise<void>} ???
     * @description ???
     * @version     1.0.0
     * @private @static
     */
    private static validateInnerRecipe;
    /**
     * @method      executeEngine
     * @param       {ExecutionOptions} [executionOptions]  Optional.
     * @returns     {Promise<ListrContext>} Resolves with a ListrContext object
     *              (basically, a type alias for "any" since Listr doesn't expose
     *              a TypeScript type for their code).
     * @description Executes the Listr Tasks that were previously compiled into
     *              this Engine. Implemented here instead of in the base class so
     *              we can fine-tune error and success handling resulting from
     *              the Listr Task execution.
     * @protected @async
     */
    protected executeEngine(executionOptions?: ExecutionOptions): Promise<ListrContext>;
    /**
     * @method      initializeActionMap
     * @returns     {Promise<void>}  ???
     * @description ???
     * @protected @async
     */
    protected initializeActionMap(): Promise<void>;
    /**
     * @method      initializePostBuildStepGroups
     * @returns     {Promise<void>}
     * @description ???
     * @protected @async
     */
    protected initializePostBuildStepGroups(): Promise<void>;
    /**
     * @method      initializePreBuildStepGroups
     * @returns     {Promise<void>}
     * @description ???
     * @protected @async
     */
    protected initializePreBuildStepGroups(): Promise<void>;
    /**
     * @method      initializeRecipeEngineContext
     * @returns     {Promise<void>}
     * @description ???
     * @protected @async
     */
    protected initializeRecipeEngineContext(): Promise<void>;
    /**
     * @method      initializeSkipActions
     * @returns     {Promise<void>} ???
     * @description ???
     * @protected @async
     */
    protected initializeSkipActions(): Promise<void>;
    /**
     * @method      initializeSkipGroups
     * @returns     {Promise<void>}
     * @description ???
     * @protected @async
     */
    protected initializeSkipGroups(): Promise<void>;
    /**
     * @method      initializeTargetOrg
     * @returns     {Promise<void>} ???
     * @description ???
     * @protected @async
     */
    protected initializeTargetOrg(): Promise<void>;
    /**
     * @method      askUserForSkipGroups
     * @returns     {Promise<Array<string>>} Resolves with an Array of strings
     *              representing the FINAL set of "Skip Groups".  This will end
     *              up determining what actually gets compiled.
     * @description Runs an Inquirer interview to present the user with a list of
     *              Step groups and the ability to select/deselect them to
     *              compose the final set of "Skip Groups".
     * @private @async
     */
    private askUserForSkipGroups;
    /**
     * @method      askUserForTargetOrgAlias
     * @returns     {Promise<string>} Resolves with the Alias of the target org
     *              as specified by the User via an Inquirer interview.
     * @description Runs an Inquirer interview to present the user with the choice
     *              of Target Orgs based on what's described in the Recipe.
     * @private @async
     */
    private askUserForTargetOrgAlias;
    /**
     * @method      confirmDemoTarget
     * @returns     {Promise<boolean>} Resolves TRUE if the user wants to start
     *              executing the Recipe against the Target Org.  FALSE if not.
     * @description Runs an Inquirer interview to ask the user if they want to
     *              start executing the Recipe against the specified Target org.
     * @version     1.0.0
     * @private @async
     */
    private confirmDemoTarget;
}
