/**
 * @file          modules/sfdx-falcon-executors/sfdx-executors.ts
 * @copyright     Vivek M. Chawla - 2018
 * @author        Vivek M. Chawla <@VivekMChawla>
 * @summary       SFDX Executor Module
 * @description   Exports functions that interact with SFDX core functionality either via shell
 *                commands or directly via internal JavaScript.
 * @version       1.0.0
 * @license       MIT
 */
import { JsonMap } from '@salesforce/ts-types';
import { SfdxFalconResult } from '../../sfdx-falcon-result';
import { Subscriber } from '../../sfdx-falcon-types';
/**
 * @interface   SfdxCommandDefinition
 * @description Represents an SFDX "Command Definition", a structure that can be compiled into
 *              a string that can be executed at the command line against the Salesforce CLI.
 */
export interface SfdxCommandDefinition {
    command: string;
    progressMsg: string;
    errorMsg: string;
    successMsg: string;
    commandArgs: string[];
    commandFlags: JsonMap;
    observer: Subscriber;
}
/**
 * @interface   ExecutorResultDetail
 * @description Represents the structure of the Detail object used by the EXECUTOR
 *              executeSfdxCommand().
 */
export interface ExecutorResultDetail {
    sfdxCommandDef: SfdxCommandDefinition;
    sfdxCommandString: string;
    stdOutParsed: JsonMap;
    stdOutBuffer: string;
    stdErrBuffer: string;
}
/**
 * @function    executeSfdxCommand
 * @param       {SfdxCommandDefinition} sfdxCommandDef  Required. Defines the command to be run.
 * @returns     {Promise<SfdxFalconExecutorResponse>} Resolves with and SFDX-Falcon Executor
 *              Response object on success.
 * @description Executes an SFDX command based on the SfdxCommandDefinition passed in.
 * @version     1.0.0
 * @public @async
 */
export declare function executeSfdxCommand(sfdxCommandDef: SfdxCommandDefinition): Promise<SfdxFalconResult>;
