/**
 * @file          modules/sfdx-falcon-types/index.d.ts
 * @copyright     Vivek M. Chawla - 2018
 * @author        Vivek M. Chawla <@VivekMChawla>
 * @summary       Collection of interfaces and types used across SFDX-Falcon modules.
 * @description   Collection of interfaces and types used across SFDX-Falcon modules.
 * @version       1.0.0
 * @license       MIT
 */
import { Connection } from '@salesforce/core';
import { AnyJson } from '@salesforce/ts-types';
import * as inquirer from 'inquirer';
import { QueryResult } from 'jsforce';
import { RequestInfo } from 'jsforce';
import { Observable } from 'rxjs';
import { Observer } from 'rxjs';
import { Subscriber } from 'rxjs';
import { Questions } from 'yeoman-generator';
import { Question } from 'yeoman-generator';
import { SfdxFalconResult } from '../sfdx-falcon-result';
import { SfdxOrgInfo } from '../sfdx-falcon-util/sfdx';
import { SfdxFalconTableData } from '../sfdx-falcon-util/ux';
/**
 * Represents the local config options for an AppX Demo project.
 * TODO: Delete this interface if not used.
 */
/**
 * Represents the configuration schema of an AppX Demo Project.
 */
/**
 * Represents the sequence options for an AppX Demo project
 * TODO: Delete this if left unused.
 */
/**
 * Represents local config settings for an APK (AppX Package) project
 * TODO: Delete this if left unused.
 */
/**
 * TODO: Delete this if left unused.
 */
/**
 * TODO: Delete this if left unused
 */
/**
 * TODO: Delete this if left unused
 */
/**
 * TODO: Delete this if left unused
 */
/**
 * TODO: Delete this if left unused.
 */
/**
 * Delete this if left unused.
 */
/**
 * Delete this if left unused.
 */
/**
 * Delete this if left unused.
 */
/**
 * Represents the status code and JSON result that is sent to the caller when SFDX-Falcon CLI Commands are run.
 */
export interface SfdxFalconJsonResponse {
    falconStatus: number;
    falconResult: AnyJson;
}
/**
 * Interface. Represents the SFDX-Falcon specific part of a project's sfdx-project.json config file.
 */
export interface SfdxFalconProjectConfig {
    developerAlias?: string;
    developerName?: string;
    projectAlias?: string;
    projectName?: string;
    projectFamily?: string;
    projectType?: string;
    defaultRecipe?: string;
    gitRemoteUri?: string;
    gitHubUrl?: string;
    projectVersion?: string;
    schemaVersion?: string;
    pluginVersion?: string;
    appxPackage?: AppxPackageProjectConfig;
    appxDemo?: AppxDemoProjectConfig;
}
/**
 * Interface. Represents the special, hidden "local config" file for an SFDX-Falcon project.
 */
export interface SfdxFalconLocalConfig {
    devHubAlias?: string;
    envHubAlias?: string;
    pkgOrgAlias?: string;
    appxPackage?: AppxPackageLocalConfig;
    appxDemo?: AppxDemoLocalConfig;
}
/**
 * Interface. Represents a "global" SFDX-Falcon configuration data structure. Not yet implmented.
 */
export interface SfdxFalconGlobalConfig {
    propertiesTBD?: any;
}
/**
 * Interface. Represents the portion of an SFDX-Falcon Project Config that is specific to ADK projects.
 */
export interface AppxDemoProjectConfig {
    demoRecipes: string[];
    partnerAlias: string;
    partnerName: string;
}
/**
 * Interface. Represents the portion of the hidden SFDX-Falcon "local config" that is specific to ADK projects.
 */
export interface AppxDemoLocalConfig {
    propertiesTBD: any;
}
/**
 * Interface. Represents the portion of an SFDX-Falcon Project Config that is specific to APK projects.
 */
export interface AppxPackageProjectConfig {
    developerRecipes: string[];
    namespacePrefix: string;
    packageName: string;
    metadataPackageId: string;
    packageVersionId: {
        stable: string;
        beta: string;
    };
}
/**
 * Interface. Represents the portion of the hidden SFDX-Falcon "local config" that is specific to APK projects.
 */
export interface AppxPackageLocalConfig {
    propertiesTBD: any;
}
/**
 * Interface. Represents the options that can be set when calling SfdxFalconProject.resolve().
 */
export interface ProjectResolutionOptions {
    resolveProjectConfig?: boolean;
    resolveLocalConfig?: boolean;
    resolveGlobalConfig?: boolean;
}
/**
 * Interface. Represents a Metadata Package (033). Can be managed or unmanaged.
 */
export interface MetadataPackage {
    Id: string;
    Name: string;
    NamespacePrefix: string;
    MetadataPackageVersions: MetadataPackageVersion[];
}
/**
 * Interface. Represents a Metadata Package Version (04t).
 */
export interface MetadataPackageVersion {
    Id: string;
    Name: string;
    MetadataPackageId: string;
    MajorVersion: number;
    MinorVersion: number;
    PatchVersion: number;
    BuildNumber: number;
    ReleaseState: string;
}
/**
 * Interface. Represents a "runnable" Listr object (ie. an object that has the run() method attached).
 */
export interface ListrObject extends Object {
    run(): Promise<any>;
}
/**
 * Interface. Represents a Listr Task object that can be executed by a Listr Task Runner.
 */
export interface ListrTask {
    title: string;
    task: ListrTaskFunction;
    skip?: boolean | ListrSkipFunction | ListrSkipCommand;
    enabled?: boolean | ListrEnabledFunction;
}
/**
 * Represents an "enabled" function for use in a Listr Task.
 */
export declare type ListrEnabledFunction = (context?: any) => boolean;
/**
 * Type. Represents a "skip" function for use in a Listr Task.
 */
export declare type ListrSkipFunction = (context?: any) => boolean | string | Promise<boolean | string>;
/**
 * Type. A built-in function of the "this task" Listr Task object that gets passed into executable task code.
 */
export declare type ListrSkipCommand = (message?: string) => void;
/**
 * Represents a "task" function for use in a Listr Task.
 */
export declare type ListrTaskFunction = (context?: ListrContext, task?: ListrTask) => void | Promise<void> | Observable<any>;
/**
 * Represents the set of "execution options" related to the use of Listr.
 */
export interface ListrExecutionOptions {
    listrContext: any;
    listrTask: any;
    observer: any;
    sharedData?: object;
}
/**
 * Represents the Listr "Context" that's passed to various functions set up inside Listr Tasks.
 */
export declare type ListrContext = any;
/**
 * Interface. Represents the Listr Context variables used by the "finalizeGit" task collection.
 */
export interface ListrContextFinalizeGit {
    gitInstalled: boolean;
    gitInitialized: boolean;
    projectFilesStaged: boolean;
    projectFilesCommitted: boolean;
    gitRemoteIsValid: boolean;
    gitRemoteAdded: boolean;
}
/**
 * Interface. Represents the Listr Context variables used by the "Package Retrieve/Extract/Convert" task collection.
 */
export interface ListrContextPkgRetExCon {
    packagesRetrieved: boolean;
    sourceExtracted: boolean;
    sourceConverted: boolean;
}
/**
 * Represents an Observable for use with Listr.
 */
export declare type ListrObservable = any;
/**
 * Type. Alias to an rxjs Observer<any> type.
 */
export declare type Observer = Observer<any>;
/**
 * Type. Alias to an rxjs Subscriber<any> type.
 */
export declare type Subscriber = Subscriber<any>;
export declare type InquirerChoice = inquirer.objects.Choice;
export declare type InquirerChoices = inquirer.objects.Choices;
export declare type InquirerQuestion = inquirer.Question;
export declare type InquirerQuestions = inquirer.Questions;
export declare type InquirerAnswers = inquirer.Answers;
/**
 * Represents an answer hash (basically AnyJson) for Yeoman/Inquirer.
 */
export interface YeomanAnswerHash {
    [key: string]: any;
}
/**
 * Represents a Yeoman/Inquirer choice object.
 */
export interface YeomanChoice {
    name: string;
    value: string;
    short: string;
    type?: string;
    line?: string;
}
/**
 * Represents a "checkbox choice" in Yeoman/Inquirer.
 */
export interface YeomanCheckboxChoice extends YeomanChoice {
    key?: string;
    checked?: boolean;
    disabled?: boolean | string | YeomanChoiceDisabledFunction;
}
/**
 * Represents the function signature for a "Disabled" function.
 */
export declare type YeomanChoiceDisabledFunction = (answers: any) => boolean | string;
/**
 * Represents what an answers hash should look like during Yeoman/Inquirer interactions
 * where the user is being asked to proceed/retry/abort something.
 */
export interface ConfirmationAnswers {
    proceed: boolean;
    restart: boolean;
    abort: boolean;
}
/**
 * Type. Defines a function that displays answers to a user.
 */
export declare type AnswersDisplay<T extends object> = (userAnswers?: T) => Promise<void | SfdxFalconTableData>;
/**
 * Type. Alias to a combination of Error or SfdxFalconResult.
 */
export declare type ErrorOrResult = Error | SfdxFalconResult;
/**
 * Interface. Represents the options that can be set by the SfdxFalconPrompt constructor.
 */
export interface PromptOptions<T extends object> {
    questions: Questions | QuestionsBuilder;
    defaultAnswers: T;
    confirmation?: Questions | QuestionsBuilder;
    invertConfirmation?: boolean;
    display?: AnswersDisplay<T>;
    context?: object;
    data?: object;
}
/**
 * Interface. Represents the options that can be set by the SfdxFalconInterview constructor.
 */
export interface InterviewOptions<T extends object> {
    defaultAnswers: T;
    confirmation?: Questions | QuestionsBuilder;
    confirmationHeader?: string;
    invertConfirmation?: boolean;
    display?: AnswersDisplay<T>;
    displayHeader?: string;
    context?: object;
    sharedData?: object;
}
/**
 * Interface. Represents the options that can be set by the InterviewGroup constructor.
 */
export interface InterviewGroupOptions<T extends object> {
    questions: Questions | QuestionsBuilder;
    confirmation?: Questions | QuestionsBuilder;
    invertConfirmation?: boolean;
    display?: AnswersDisplay<T>;
    when?: ShowInterviewGroup;
    abort?: AbortInterview;
    title?: string;
}
/**
 * Interface. Represents a set of status indicators for an SfdxFalconInterview.
 */
export interface InterviewStatus {
    aborted?: boolean;
    completed?: boolean;
    reason?: string;
}
/**
 * Type alias defining a function that checks whether an Interview should be aborted.
 */
export declare type AbortInterview = (groupAnswers: InquirerAnswers, userAnswers?: InquirerAnswers) => boolean | string;
/**
 * Type alias defining a function that can be used to determine boolean control-flow inside an Interview.
 */
export declare type InterviewControlFunction = (userAnswers: InquirerAnswers, sharedData?: object) => boolean | Promise<boolean>;
/**
 * Type alias defining a function or simple boolean that checks whether an Interview Group should be shown.
 */
export declare type ShowInterviewGroup = boolean | InterviewControlFunction;
/**
 * Function type alias defining a function that returns Inquirer Questions.
 */
export declare type QuestionsBuilder = () => Questions;
/**
 * Alias to the Questions type from yeoman-generator. This is the "official" type for SFDX-Falcon.
 */
export declare type Questions = Questions;
/**
 * Alias to the Question type from yeoman-generator. This is the "official" type for SFDX-Falcon.
 */
export declare type Question = Question;
/**
 * Type. Represents either an Org Alias or a JSForce Connection.
 */
export declare type AliasOrConnection = string | Connection;
/**
 * Interface. Represents a resolved (active) JSForce connection to a Salesforce Org.
 */
export interface ResolvedConnection {
    connection: Connection;
    orgIdentifier: string;
}
/**
 * Interface. Represents information needed to make a REST API request via a JSForce connection.
 */
export interface RestApiRequestDefinition {
    aliasOrConnection: string | Connection;
    request: RequestInfo;
    options?: {
        any: any;
    };
}
/**
 * Type. Alias to a Map with string keys and MetadataPackageVersion values.
 */
export declare type PackageVersionMap = Map<string, MetadataPackageVersion[]>;
/**
 * Type. Alias to the JSForce definition of QueryResult.
 */
export declare type QueryResult<T> = QueryResult<T>;
/**
 * Interface. Represents the data returned by the sfdx force:org:list command.
 */
export interface RawSfdxOrgInfo {
    alias: string;
    username: string;
    orgId: string;
    connectedStatus: string;
    isDevHub: boolean;
}
/**
 * Type. Alias for a Map with string keys holding SfdxOrgInfo values.
 */
export declare type SfdxOrgInfoMap = Map<string, SfdxOrgInfo>;
/**
 * Interface. Represents the subset of Org Information that's relevant to SFDX-Falcon logic.
 */
export interface SfdxOrgInfoSetup {
    alias: string;
    username: string;
    orgId: string;
    connectedStatus: string;
    isDevHub: boolean;
    metadataPackageResults?: QueryResult<MetadataPackage>;
}
/**
 * Enum. Represents the various CLI log level flag values.
 */
export declare enum SfdxCliLogLevel {
    TRACE = "trace",
    DEBUG = "debug",
    INFO = "info",
    WARN = "warn",
    ERROR = "error",
    FATAL = "fatal"
}
/**
 * Interface. Represents the result of a call to shell.execL().
 */
export interface ShellExecResult {
    code?: number;
    stdout?: string;
    stderr?: string;
    message?: string;
    resolve?: boolean;
}
/**
 * Interface. Represents the REST response provided for an Object Describe.
 */
export interface ObjectDescribe {
    activateable?: boolean;
    createable?: boolean;
    custom?: boolean;
    customSetting?: boolean;
    deletable?: boolean;
    deprecatedAndHidden?: boolean;
    feedEnabled?: boolean;
    hasSubtypes?: boolean;
    isSubtype?: boolean;
    keyPrefix?: string;
    label?: string;
    labelPlural?: string;
    layoutable?: boolean;
    mergeable?: boolean;
    mruEnabled?: boolean;
    name?: string;
    queryable?: boolean;
    replicateable?: boolean;
    retrieveable?: boolean;
    searchable?: boolean;
    triggerable?: boolean;
    undeletable?: boolean;
    updateable?: boolean;
    urls?: any;
}
/**
 * Interface. Represents a baseline SObject.
 */
export interface SObject {
    id?: string;
    name?: string;
}
/**
 * Interface. Represents the Salesforce Profile SObject.
 */
export declare type Profile = SObject;
/**
 * Interface. Represents the Salesforce PermissionSetAssignment SObject.
 */
export interface PermissionSetAssignment extends SObject {
    PermissionSetId: string;
    AssigneeId: string;
}
/**
 * Interface. Represents the Salesforce User SObject.
 */
export interface User extends SObject {
    username?: string;
}
/**
 * Type. Alias for an array of objects that may have "Id" and "Name" properties.
 */
export declare type SObjectFindResult = Array<{
    Id?: string;
    Name?: string;
}>;
