import { JsonMap } from '@salesforce/ts-types';
/**
 * @function    createUniqueUsername
 * @param       {string}  baseUsername  The starting point for the username.  It should already be
 *                                      in the form of an email, eg 'name@domain.org'.
 * @returns     {string}  Returns the baseUsername with a pseudo-uuid appended to the end.
 * @description Given a base username to start with (eg. 'name@domain.org'), returns what should be
 *              a globally unique username with a pseudo-uuid appended the end of the username base.
 * @version     1.0.0
 * @public
 */
export declare function createUniqueUsername(baseUsername: string): string;
/**
 * @function    readConfigFile
 * @param       {string}  rootFolder  Required. The root folder where the config file is stored.
 * @param       {string}  filename    Required. The name of the config file.
 * @returns     {Promise<any>}    Resolves by returning the config file contents as a JS Object.
 * @description Given a path and filename, attempts to load the contents of the config file and
 *              convert them into a JavaScript Object.
 * @version     1.0.0
 * @public @async
 */
export declare function readConfigFile(rootFolder: string, filename: string): Promise<JsonMap>;
/**
 * @function    safeParse
 * @param       {any} contentToParse  Required. The content to be parsed.
 * @returns     {object}  A JavaScript object based on the content to parse.
 * @description Given any content to parse, returns a JavaScript object based on that content. If
 *              the content is not parseable, it is returned as an object with one key: unparsed.
 * @public
 */
export declare function safeParse(contentToParse: string): JsonMap;
/**
 * @function    findJson
 * @param       {string} contentToSearch  Required. A string buffer that may contain JSON.
 * @returns     {JsonMap}  A parsed JavaScript object found in the string buffer, or NULL.
 * @description Given any string buffer, search that buffer to find a single JSON object. If
 *              a parseable object is found, it is returned as an object. Otherwise returns NULL.
 * @public
 */
export declare function findJson(contentToSearch: string): JsonMap;
