import { ListrObject } from '../sfdx-falcon-types';
import { ListrTask } from '../sfdx-falcon-types';
/**
 * @function    addGitRemote
 * @param       {string}  targetDir Required. Location where the git command will be run
 * @param       {string}  gitRemoteUri  Required. URI of the Git Remote to be added as origin.
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that adds the provided Git Remote as the
 *              origin remote.
 * @public
 */
export declare function addGitRemote(targetDir: string, gitRemoteUri: string): ListrTask;
/**
 * @function    buildDevHubAliasList
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that takes a list of identified Dev Hubs
 *              and uses it to create an Inquirer-compatible "choice list". This function must be
 *              executed using the call() method because it relies on the caller's "this" context
 *              to properly function.
 * @public
 */
export declare function buildDevHubAliasList(): ListrTask;
/**
 * @function    buildEnvHubAliasList
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that takes a list of identified Environment
 *              Hubs and uses it to create an Inquirer-compatible "choice list". This function must
 *              be executed using the call() method because it relies on the caller's "this" context
 *              to properly function.
 * @public
 */
export declare function buildEnvHubAliasList(): ListrTask;
/**
 * @function    buildPkgOrgAliasList
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that takes a list of identified Packaging
 *              Orgs and uses it to create an Inquirer-compatible "choice list". This function must
 *              be executed using the call() method because it relies on the caller's "this" context
 *              to properly function.
 * @public
 */
export declare function buildPkgOrgAliasList(): ListrTask;
/**
 * @function    cloneGitRemote
 * @param       {string}  gitRemoteUri Required. ???
 * @param       {string}  targetDirectory Required. ???
 * @param       {string}  [gitCloneDirectory='']  Required. ???
 * @returns     {ListrObject}  A "runnable" Listr Object
 * @description Returns a "runnable" Listr Object that attempts to clone the Git Repository referred
 *              to by the provided Git Remote URI.
 * @public
 */
export declare function cloneGitRemote(gitRemoteUri: string, targetDirectory: string, gitCloneDirectory?: string): ListrObject;
/**
 * @function    commitProjectFiles
 * @param       {string}  targetDir Required.
 * @param       {string}  commitMessage Required.
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that commits whatever is in the Target
 *              Directory, using the commitMessage parameter for the Commit Message.
 * @public
 */
export declare function commitProjectFiles(targetDir: string, commitMessage: string): ListrTask;
/**
 * @function    fetchAndConvertManagedPackage
 * @param       {string}  aliasOrUsername Required. The alias or username associated with a
 *              packaging org that the Salesforce CLI is currently connected to.
 * @param       {string}  packageName Required. The name of the desired managed package.
 * @param       {string}  projectDir  Required. The root of the Project Directory
 * @param       {string}  packageDir  Required. Name of the default package directory, located
 *              inside of "projectDir/sfdx-source/"
 * @returns     {ListrObject}  A "runnable" Listr Object
 * @description Returns a "runnable" Listr Object that attempts to retrieve, extract, and convert
 *              the metadata for the specified package from the specified org. The converted
 *              metadata source will be saved to the Package Directory specified by the caller.
 * @public
 */
export declare function fetchAndConvertManagedPackage(aliasOrUsername: string, packageName: string, projectDir: string, packageDir: string): ListrObject;
/**
 * @function    finalizeGit
 * @param       {string}  targetDirectory Required. Directory that will be initialized with Git.
 * @param       {string}  [gitRemoteUri]  Optional. URI of the remote that should be associated
 *              with the repository that we're going to initialize.
 * @returns     {ListrObject}  A "runnable" Listr Object
 * @description Returns a "runnable" Listr Object that initializes Git in the Target Directory, then
 *              connects that repo to the remote specified by the Git Remote URI.
 * @public
 */
export declare function finalizeGit(targetDir: string, gitRemoteUri?: string, gitCommitMsg?: string): ListrObject;
/**
 * @function    gitInitTasks
 * @returns     {ListrObject}  A "runnable" Listr Object
 * @description Returns a Listr-compatible Task Object that verifies the presence of the Git
 *              executable in the local environment.
 * @public
 */
export declare function gitInitTasks(): ListrObject;
/**
 * @function    gitRuntimeCheck
 * @param       {string}  dbgNs Required. Debug namespace. Ensures proper debug output.
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that verifies the presence of the Git
 *              executable in the local environment.
 * @public
 */
export declare function gitRuntimeCheck(): ListrTask;
/**
 * @function    identifyDevHubs
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that takes a list of raw SFDX Org Infos
 *              and searches them to identify any Dev Hubs.  This function must be invoked using
 *              the call() method because it relies on the caller's "this" context to function.
 * @public
 */
export declare function identifyDevHubs(): ListrTask;
/**
 * @function    identifyEnvHubs
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that takes a list of raw SFDX Org Infos
 *              and searches them to identify any Environment Hubs.  This function must be invoked
 *              using the call() method because it relies on the caller's "this" context to function.
 * @public
 */
export declare function identifyEnvHubs(): ListrTask;
/**
 * @function    identifyPkgOrgs
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that takes a list of raw SFDX Org Infos
 *              and searches them to identify any Packaging Orgs.  This function must be invoked
 *              using the call() method because it relies on the caller's "this" context to function.
 * @public
 */
export declare function identifyPkgOrgs(): ListrTask;
/**
 * @function    initializeGit
 * @param       {string}  targetDir
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that initializes Git in the target directory.
 * @public
 */
export declare function initializeGit(targetDir: string): ListrTask;
/**
 * @function    packagedMetadataFetch
 * @param       {string}  aliasOrUsername Required. The alias or username associated with a current
 *              Salesforce CLI connected org.
 * @param       {string[]}  packageNames  Required. String array containing the names of all
 *              packages that should be retrieved.
 * @param       {string}  retrieveTargetDir Required. The root of the directory structure where
 *              the retrieved .zip or metadata files are put.
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that attempts to retrieve the metadata
 *              for the specified package from the specified org. The metadata will be saved to
 *              the local filesystem at the location specified by the caller.
 * @public
 */
export declare function packagedMetadataFetch(aliasOrUsername: string, packageNames: string[], retrieveTargetDir: string): ListrTask;
/**
 * @function    reValidateGitRemote
 * @param       {string}  gitRemoteUri
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that attempts to re-validate the presence of
 *              a Git Remote at the Git Remote URI provided.
 * @public
 */
export declare function reValidateGitRemote(gitRemoteUri: string): ListrTask;
/**
 * @function    scanConnectedOrgs
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that scans the orgs that are connected to
 *              (ie. authenticated to) the local SFDX environment. The raw list of these orgs is
 *              added to the Listr Context var so it's available to subsequent Listr tasks.
 * @public
 */
export declare function scanConnectedOrgs(): ListrTask;
/**
 * @function    sfdxInitTasks
 * @returns     {ListrObject}  A "runnable" Listr Object
 * @description Returns a Listr-compatible Task Object that contains a number of sub-tasks which
 *              inspect the connected orgs in the local SFDX environment and build Inquirer "choice
 *              lists" with them.
 * @public
 */
export declare function sfdxInitTasks(): ListrObject;
/**
 * @function    stageProjectFiles
 * @param       {string}  targetDir
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that stages (git -A) ALL files in the target
 *              directory.
 * @public
 */
export declare function stageProjectFiles(targetDir: string): ListrTask;
/**
 * @function    validateGitRemote
 * @param       {string}  gitRemoteUri  Required. URI of the remote Git repository being validated.
 * @returns     {ListrTask}  A Listr-compatible Task Object
 * @description Returns a Listr-compatible Task Object that validates the presence of and access to
 *              the Git remote at the provided Git Remote URI.
 * @public
 */
export declare function validateGitRemote(gitRemoteUri?: string): ListrTask;
