import { SfdxOrgInfo } from '../../modules/sfdx-falcon-util/sfdx';
import { StatusMessage } from '../../modules/sfdx-falcon-util/ux';
import { YeomanChoice } from '../../modules/sfdx-falcon-types';
/**
 * @class       YeomanSeparator
 * @summary     Separator object for use when creating Yeoman Lists.
 * @description Separator object for use when creating Yeoman Lists. This is essentially a wrapper
 *              for an Inquirer Separator since Yeoman uses Inquirer to query the user.
 * @public
 */
export declare class YeomanSeparator {
    name: string;
    value: string;
    short: string;
    type: string;
    line?: string;
    constructor(separatorLine?: string);
}
/**
 * @class       GeneratorStatus
 * @summary     Status tracking object for use with Yeoman Generators.
 * @description Specialized object used by Yeoman Generators to track the running state of the
 *              Generator (eg. aborted/completed) as well as collection of status messages that can
 *              be used to print out a final "status report" when the Generator is complete.
 * @public
 */
export declare class GeneratorStatus {
    aborted: boolean;
    completed: boolean;
    running: boolean;
    messages: StatusMessage[];
    /**
     * @constructs  GeneratorStatus
     * @description Constructs a GeneratorStatus object.
     * @public
     */
    constructor();
    /**
     * @method      start
     * @param       {statusMessage} statusMessage Optional. Allows the caller to
     *              set an initial status message upon starting this object.
     * @returns     {void}
     * @description Starts this Generator Status object by changing the "running"
     *              property to TRUE.
     * @public
     */
    start(statusMessage?: StatusMessage): void;
    /**
     * @method      abort
     * @param       {statusMessage} statusMessage Optional. Allows the caller to
     *              set an initial status message upon aborting this object.
     * @returns     {void}
     * @description Aborts this Generator Status object by changing the "running"
     *              property to FALSE and the "aborted" property to TRUE.
     * @public
     */
    abort(statusMessage: StatusMessage): void;
    /**
     * @method      complete
     * @param       {statusMessage[]} statusMessages  Optional. Allows the caller to
     *              set an array of status message upon completing this object.
     * @returns     {void}
     * @description Completes this Generator Status object by changing the "running"
     *              property to FALSE and the "completed" property to TRUE.
     * @public
     */
    complete(statusMessages?: StatusMessage[]): void;
    /**
     * @method      addMessage
     * @param       {statusMessage} statusMessage  Required. Status Message to add
     *              to this Generator Status object.
     * @returns     {void}
     * @description Given a Status Message object, adds that Status Message to the
     *              messages array of this Generator Status object.
     * @public
     */
    addMessage(statusMessage: StatusMessage): void;
    /**
     * @method      printStatusMessages
     * @returns     {void}
     * @description Prints the Status Messages of this object using the UX helper
     *              function printStatusMessages().
     * @public
     */
    printStatusMessages(): void;
}
/**
 * @function    buildOrgAliasChoices
 * @param       {string}  alias Required.
 * @param       {string}  username  Required.
 * @param       {number}  padLength Required.
 * @returns     {YeomanChoice}
 * @description Given an arrray of SfdxOrgInfo objects, builds a fully formed array of Yeoman Choice
 *              objects.  This can be given to Yeoman (or any Inquirer-based interview system) in
 *              order to display a list of choices to the user.
 * @public
 */
export declare function buildOrgAliasChoices(sfdxOrgInfos: SfdxOrgInfo[]): YeomanChoice[];
/**
 * @function    filterLocalPath
 * @param       {string}  localPath Required.
 * @returns     {string}  A resolved version of the path string provided by localPath.
 * @description Yeoman filter function which takes a local Path value and resolves it by using
 *              path.resolve(), and then returns that value.
 */
export declare function filterLocalPath(localPath: string): string;
