/**
 * @function    gitRemoteUri
 * @param       {string}  gitRemoteUri  Required. A URI for a Git Remote
 * @param       {RegExp}  [acceptedProtocols] Optional. RegExp that matches only certain protocols.
 * @returns     {boolean} True if gitRemoteUri is a valid Git Remote URI.
 * @description Core validation function for ensuring well-formed Git Remote URIs.
 *              See https://git-scm.com/docs/git-clone for detailed rules.
 * @public
 */
export declare function validateGitRemoteUri(gitRemoteUri: string, acceptedProtocols?: RegExp): boolean;
/**
 * @function    validateLocalPath
 * @param       {string}  pathString  Required.
 * @returns     {boolean} FALSE if pathString contains invalid chars
 * @description Ensures that the provided pathString does not have any invalid characters.  This
 *              doesn't cover everything, but it's a stab at trying to stop some chars that could
 *              be trouble on Windows machines.
 * @public
 */
export declare function validateLocalPath(pathString: string): boolean;
/**
 * @function    validateStandardAlias
 * @param       {string}  alias Required.
 * @param       {number}  [maxLength] Optional.
 * @returns     {boolean} FALSE if alias contains invalid chars or is too long (optionally).
 * @description Ensures that the provided alias does not have any invalid characters.  Optionally,
 *              can also check for maximum length.
 * @public
 */
export declare function validateStandardAlias(alias: string, maxLength?: number): boolean;
/**
 * @function    validateStandardName
 * @param       {string}  name Required.
 * @param       {number}  [maxLength] Optional.
 * @returns     {boolean} FALSE if name contains invalid chars or is too long (optionally).
 * @description Ensures that the provided name does not have any invalid characters.  Optionally,
 *              can also check for maximum length.
 * @public
 */
export declare function validateStandardName(name: string, maxLength?: number): boolean;
