/**
 * @function    gitRemoteUri
 * @param       {string}  userInput Required.
 * @returns     {boolean|string}  TRUE if input is valid. Error message STRING if invalid.
 * @description Validate that the user-provided string is a properly formed URI for a Git Remote.
 *              Only the syntax of the URI is being checked, not whether the repo exists or not.
 * @public
 */
export declare function gitRemoteUri(userInput: string): boolean | string;
/**
 * @function    targetPath
 * @param       {string}  userInput Required.
 * @returns     {boolean|string}  TRUE if input is valid. Error message STRING if invalid.
 * @description Validate that the user-provided string is a valid local path, based on the running
 *              user's environment (ie. Mac/PC/Linux)
 * @public
 */
export declare function targetPath(userInput: string): boolean | string;
/**
 * @function    projectName
 * @param       {string}  userInput Required.
 * @returns     {boolean|string}  TRUE if input is valid. Error message STRING if invalid.
 * @description Validate that the user-provided string is...
 * @public
 */
export declare function projectName(userInput: string): boolean | string;
/**
 * @function    standardAlias
 * @param       {string}  userInput Required.  User input from a Yeoman/Inquirer interview.
 * @returns     {boolean|string}  TRUE if input is valid. Error message STRING if invalid.
 * @description Validate that the user-provided string is 1-15 characters long and includes only
 *              letters (a-Z), numbers (0-9), hyphens (-), and underscores (_).
 * @public
 */
export declare function standardAlias(userInput: string): boolean | string;
/**
 * @function    standardName
 * @param       {string}  userInput Required.  User input from a Yeoman/Inquirer interview.
 * @returns     {boolean|string}  TRUE if input is valid. Error message STRING if invalid.
 * @description Validate that the user-provided string is less than ?? chars long
 * @public
 */
export declare function standardName(userInput: string): boolean | string;
